#include "Class-definitions.h"
#include "rtxsrc/rtxCommon.h"

#include <stdio.h>
#include <stdlib.h>

int main (int argc, char** argv)
{
   UL_DCCH_Message* pvalue;
   OSCTXT     ctxt, *pctxt;
   OSOCTET*   msgptr;
   FILE*      fp;
   int        i, len, stat;
   OSBOOL     aligned = FALSE;
   OSBOOL     trace = TRUE;
   OSBOOL     verbose = FALSE;
   const char* filename = "message.dat";

   /* Process command line arguments */
   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = TRUE;
         else if (!strcmp (argv[i], "-u")) aligned = FALSE;
         else if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer -a | -u [ -v ] [ -o <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 1;
         }
      }
   }

   /* Initialize context structure */
   stat = rtInitContext (&ctxt);
   if (stat != 0) {
      rtxErrPrint (&ctxt);
      return stat;
   }
   rtxSetDiag (&ctxt, verbose);
   pu_setBuffer (&ctxt, 0, 0, aligned);
   pu_setTrace (&ctxt, trace);

   pctxt = &ctxt;
   pvalue = rtxMemAllocType (pctxt, UL_DCCH_Message);
   asn1Init_UL_DCCH_Message (pvalue);

   pvalue->m.integrityCheckInfoPresent = 1;

   pvalue->integrityCheckInfo.messageAuthenticationCode.numbits = 32;
   pvalue->integrityCheckInfo.messageAuthenticationCode.data[0] = 0x30;
   pvalue->integrityCheckInfo.messageAuthenticationCode.data[1] = 0x3b;
   pvalue->integrityCheckInfo.messageAuthenticationCode.data[2] = 0x66;
   pvalue->integrityCheckInfo.messageAuthenticationCode.data[3] = 0x6a;

   pvalue->integrityCheckInfo.rrc_MessageSequenceNumber = 8;

   pvalue->message.t =  T_UL_DCCH_MessageType_mbmsModificationRequest;
   pvalue->message.u.mbmsModificationRequest = 
      rtxMemAllocTypeZ (pctxt, MBMSModificationRequest);

   pvalue->message.u.mbmsModificationRequest->
      m.mbms_PreferredFreqRequestPresent = 1;
   {
      static const OSOCTET testdata[] = { 
      0x67, 0x06, 0x7f };
      OSCRTLMEMCPY ((void*)pvalue->message.u.mbmsModificationRequest->
                    mbms_PreferredFreqRequest.serviceIdentity, testdata, 3);
   }

   pvalue->message.u.mbmsModificationRequest->
      mbms_PreferredFreqRequest.plmn_Identity.t =  
      T_MBMS_ServiceIdentity_r6_plmn_Identity_sameAsMIB_PLMN_Id;

   pvalue->message.u.mbmsModificationRequest->
      m.rb_InformationReleaseListPresent = 1;

   pvalue->message.u.mbmsModificationRequest->rb_InformationReleaseList.n = 6;
   pvalue->message.u.mbmsModificationRequest->
      rb_InformationReleaseList.elem[0] = 17;
   pvalue->message.u.mbmsModificationRequest->
      rb_InformationReleaseList.elem[1] = 15;
   pvalue->message.u.mbmsModificationRequest->
      rb_InformationReleaseList.elem[2] = 20;
   pvalue->message.u.mbmsModificationRequest->
      rb_InformationReleaseList.elem[3] = 16;
   pvalue->message.u.mbmsModificationRequest->
      rb_InformationReleaseList.elem[4] = 3;
   pvalue->message.u.mbmsModificationRequest->
      rb_InformationReleaseList.elem[5] = 28;

   pvalue->message.u.mbmsModificationRequest->
      m.v6b0NonCriticalExtensionsPresent = 1;

   pvalue->message.u.mbmsModificationRequest->
      v6b0NonCriticalExtensions.mbmsModificationRequest_v6b0ext.
      mbmsSelectedServiceInfo.status.t =  T_MBMS_SelectedServiceInfo_status_some;
   pvalue->message.u.mbmsModificationRequest->
      v6b0NonCriticalExtensions.mbmsModificationRequest_v6b0ext.
      mbmsSelectedServiceInfo.status.u.some = 
      rtxMemAllocTypeZ (pctxt, MBMS_SelectedServicesListFull);

   {
      OSRTDList* plist = pvalue->message.u.mbmsModificationRequest->
         v6b0NonCriticalExtensions.mbmsModificationRequest_v6b0ext.
         mbmsSelectedServiceInfo.status.u.some;
      MBMS_ServiceIdentity_r6* pdata;
      rtxDListInit (plist);

      pdata = rtxMemAllocTypeZ (pctxt, MBMS_ServiceIdentity_r6);
      asn1Init_MBMS_ServiceIdentity_r6_plmn_Identity (&pdata->plmn_Identity);
      {
         static const OSOCTET testdata[] = { 0x33, 0x66, 0xac };
         OSCRTLMEMCPY ((void*)pdata->serviceIdentity, testdata, 3);
      }

      pdata->plmn_Identity.t =  T_MBMS_ServiceIdentity_r6_plmn_Identity_other;
      pdata->plmn_Identity.u.other = 
         rtxMemAllocTypeZ (pctxt, MBMS_ServiceIdentity_r6_plmn_Identity_other);
      pdata->plmn_Identity.u.other->t =  
         T_MBMS_ServiceIdentity_r6_plmn_Identity_other_sameAsMIB_MultiPLMN_Id;

      pdata->plmn_Identity.u.other->u.sameAsMIB_MultiPLMN_Id = 1;

      rtxDListAppend (pctxt, plist, (void*)pdata);
   }

   pvalue->message.u.mbmsModificationRequest->
      v6b0NonCriticalExtensions.m.v6f0NonCriticalExtensionsPresent = 1;

   pvalue->message.u.mbmsModificationRequest->
      v6b0NonCriticalExtensions.v6f0NonCriticalExtensions.
      mbmsModificationRequest_v6f0ext.m.
      mbmsSupportOfServiceChangeForAPtpRBPresent = 1;

   pvalue->message.u.mbmsModificationRequest->v6b0NonCriticalExtensions.
      v6f0NonCriticalExtensions.mbmsModificationRequest_v6f0ext.
      mbmsSupportOfServiceChangeForAPtpRB = 
      MBMSModificationRequest_v6f0ext_IEs_mbmsSupportOfServiceChangeForAPtpRB_true;

   pvalue->message.u.mbmsModificationRequest->v6b0NonCriticalExtensions.
      v6f0NonCriticalExtensions.m.nonCriticalExtensionsPresent = 1;

   if (trace) {
      printf ("The following record will be encoded:\n");
      asn1Print_UL_DCCH_Message ("data", pvalue);
      printf ("\n");
   }

   /* Encode */
   if ((stat = asn1PE_UL_DCCH_Message (&ctxt, pvalue))== 0) {
      if (trace) {
         printf ("Encoding was successful\n");
         printf ("Hex dump of encoded record:\n");
         pu_hexdump (&ctxt);
         printf ("Binary dump:\n");
         pu_bindump (&ctxt, "data");
      }
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      rtFreeContext (&ctxt);
      return -1;
   }
   msgptr = pe_GetMsgPtr (&ctxt, &len);
   /* Write the encoded message out to the output file */
   if ((fp = fopen (filename, "wb"))) {
      fwrite (msgptr, 1, len, fp);
      fclose (fp);
   }
   else {
      printf ("Error opening %s for write access\n", filename);
      rtFreeContext (&ctxt);
      return -1;
   }

   rtFreeContext (&ctxt);
   return 0;
}
